"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var system_date_1 = require("../system-date");
var files_utils_1 = require("../utils/files-utils");
var istanbul_unique_id_converter_1 = require("./istanbul-unique-id-converter");
function resolveNewId(uniqueIdKey, moduleName, relativePath, coverageObject, logger) {
    moduleName = path_1.normalize(moduleName);
    var moduleData = resolveModule(coverageObject, moduleName, logger);
    if (!moduleData) {
        return;
    }
    if (!moduleData.uniqueIdsMap) {
        logger.debug("building uniqueids map for " + moduleData.path);
        var before_1 = system_date_1.getSystemDateValueOf();
        logger.debug("file " + moduleData.path + " not contains uniqueIdMaps, trying to reload");
        createSLMapping(moduleData.path, relativePath, coverageObject, logger);
        var after = system_date_1.getSystemDateValueOf();
        logger.debug("*******************************create unique id map ******************************************");
        logger.debug(after - before_1);
        logger.debug("*******************************create unique id map ******************************************");
    }
    var newUniqueId = moduleData.uniqueIdsMap[uniqueIdKey];
    if (!newUniqueId) {
        logger.error("could not generate new uniqueId for '" + uniqueIdKey + "'");
    }
    return newUniqueId;
}
exports.resolveNewId = resolveNewId;
function resolveModule(coverageObject, moduleName, logger) {
    var withLeftSlashes = files_utils_1.FilesUtils.adjustPathSlashes(moduleName);
    var moduleData = coverageObject[moduleName] || coverageObject[withLeftSlashes];
    if (!moduleData) {
        logger.warn("Coverage object not contains data for " + moduleName + " or " + withLeftSlashes);
    }
    return moduleData;
}
function createSLMapping(fullPath, relativePath, coverageObject, logger) {
    var module = coverageObject[fullPath];
    var converter = new istanbul_unique_id_converter_1.IstanbulUniqueIdConverter(relativePath, logger);
    converter.createElementsArray(module);
    converter.process();
    coverageObject[fullPath].uniqueIdsMap = converter.uniqueIdsMap;
}
//# sourceMappingURL=new-id-resolver.js.map