import Logger = require("bunyan");
export interface IGlobalErrorHandler {
    setLastError(e: Error, message: string): any;
}
export interface IMap<T> {
    [key: string]: T;
}
export interface IHandlerParams {
    buildArguments: {
        workspacepath: string;
        sendContributors: boolean;
    };
    commitToIndexMap: IMap<number>;
    emailToIndexMap: IMap<number>;
}
export interface IContributorData {
    contributorEmail: string;
    contributorName: string;
}
export interface IContributors {
    contributors: IContributorData[];
    emailToIndexMap: IMap<number>;
}
export interface ICommitLogEntry {
    commit: string;
    contributorIndex?: number;
    authorName: string;
    authorEmail: string;
    commiterName: string;
    commiterEmail: string;
    authorDate: number;
    commiterDate: number;
    title: string;
}
export interface IBranchHistory {
    commitHistory: string[];
    commitLog: ICommitLogEntry[];
    commitToIndexMap: IMap<string>;
}
export interface IGitDiffFileStatus {
    path: string;
    op: "added" | "deleted" | "modified" | "renamed" | "renamed-modified";
}
export interface IScm {
    detectCommitVersion(handlerParams: IHandlerParams, logger: Logger): Promise<string>;
    getBranchHistory(handlerParams: IHandlerParams, logger: Logger): Promise<IBranchHistory>;
    getContributors(handlerParams: IHandlerParams, logger: Logger): Promise<IContributors>;
    getCommitsPerFile(handlerParams: IHandlerParams, logger: Logger): Promise<IMap<number[]>>;
    getRepositoryUrl(handlerParams: IHandlerParams, logger: Logger): Promise<string>;
    getRootDirectory(handlerParams: IHandlerParams, logger: Logger): Promise<string>;
    getFiles(handlerParams: IHandlerParams, logger: Logger): Promise<string[]>;
    getCommitOnlyHistory(historyLength: number, handlerParams: IHandlerParams, logger: Logger): Promise<string[]>;
    getGitDiffs(refCommit: string, handlerParams: IHandlerParams, logger: Logger): Promise<IGitDiffFileStatus[]>;
}
