"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var child_process_1 = require("child_process");
var git_format_builder_1 = require("./git-format-builder");
var GIT_ROOT_DIR_COMMAND = "rev-parse --show-toplevel";
var GIT_LS_FILES_COMMAND = "ls-tree --full-tree -r --name-only HEAD";
var MAX_BUFFER_VALUE = 1024 * 1024 * 200; //200 Mb, instead of 200Kb
var Git = /** @class */ (function () {
    function Git(buildArgs, globalErrorHandler) {
        this.buildArgs = buildArgs;
        this.globalErrorHandler = globalErrorHandler;
    }
    Git.prototype.detectCommitVersion = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, commit;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        log = logger.child({ methodName: "detectCommitVersion" });
                        return [4 /*yield*/, this.executeGit("log -n 1 --pretty=format:%H", handlerParams, log)];
                    case 1:
                        commit = _a.sent();
                        return [2 /*return*/, commit];
                }
            });
        });
    };
    Git.prototype.getBranchHistory = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var buildArgs, log, historyLength, format, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        buildArgs = this.getBuildArgs();
                        log = logger.child({ methodName: "getBranchHistory" });
                        historyLength = this.getHistoryLength(buildArgs, log);
                        format = new git_format_builder_1.GitFormatBuilder().withCommit().withAuthorName().withAuthorEmail().withCommiterName().withCommiterEmail()
                            .withAuthorDate().withCommiterDate().withTitle().build();
                        return [4 /*yield*/, this.executeGit("log -n " + historyLength + " --format=" + format, handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, this.parseBranchHistoryOutput(stdout, handlerParams, log)];
                }
            });
        });
    };
    Git.prototype.getCommitsPerFile = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var buildArgs, log, historyLength, commitToIndexMap, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        buildArgs = this.getBuildArgs();
                        log = logger.child({ methodName: "getCommitsPerFile" });
                        historyLength = this.getHistoryLength(buildArgs, log);
                        commitToIndexMap = handlerParams.commitToIndexMap;
                        if (!commitToIndexMap) {
                            logger.error("Cannot get commits for file, Commits log map is null or undefined");
                            return [2 /*return*/, {}];
                        }
                        return [4 /*yield*/, this.executeGit("log -" + historyLength + " --name-only --pretty=format:hash:%H", handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, this.parseCommitsPerFileOutput(stdout, commitToIndexMap, log)];
                }
            });
        });
    };
    Git.prototype.getRepositoryUrl = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, stdout, repositoryUrl, parsedLog;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        log = logger.child({ methodName: "getRepositoryUrl" });
                        return [4 /*yield*/, this.executeGit("remote -v", handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        repositoryUrl = '';
                        if (stdout.toString()) {
                            parsedLog = this.logParse(stdout.toString());
                            //NOTE: If there are multiple (fetch) remotes, it would be best to pick the line with that has "origin" as the first column (before the \t), if one exists.
                            //Anyway we should also warn that there were multiple remotes detected            
                            repositoryUrl = parsedLog[0].split('\t').map(function (l) { return l.trim(); })[1].replace(' (fetch)', '');
                        }
                        return [2 /*return*/, repositoryUrl];
                }
            });
        });
    };
    Git.prototype.getRootDirectory = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        log = logger.child({ methodName: "getRootDirectory" });
                        return [4 /*yield*/, this.executeGit(GIT_ROOT_DIR_COMMAND, handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, stdout.toString().trim()];
                }
            });
        });
    };
    Git.prototype.getFiles = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        log = logger.child({ methodName: "getFiles" });
                        return [4 /*yield*/, this.executeGit(GIT_LS_FILES_COMMAND, handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, stdout.trim().split("\n").map(function (file) { return file.trim(); }).filter(function (file) { return file.length > 0; })];
                }
            });
        });
    };
    Git.prototype.getContributors = function (handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var buildArgs, log, historyLength, format, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        buildArgs = this.getBuildArgs();
                        log = logger.child({ methodName: "getContributors" });
                        historyLength = this.getHistoryLength(buildArgs, log);
                        format = new git_format_builder_1.GitFormatBuilder().withAuthorName("contributorName").withAuthorEmail("contributorEmail").build();
                        return [4 /*yield*/, this.executeGit("log -n " + historyLength + " --all --format=" + format, handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, this.parseContributorsOutput(stdout, log)];
                }
            });
        });
    };
    Git.prototype.getGitDiffs = function (refCommit, handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, stdout, parsedLog;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        log = logger.child({ methodName: "getGitDiffs" });
                        return [4 /*yield*/, this.executeGit("diff " + refCommit + " --name-status -M100", handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        parsedLog = this.parseGitDiffs(stdout.toString()).filter(function (d) { return d.op != "unknown" && d.path; });
                        return [2 /*return*/, parsedLog];
                }
            });
        });
    };
    Git.prototype.getCommitOnlyHistory = function (historyLength, handlerParams, logger) {
        return __awaiter(this, void 0, void 0, function () {
            var log, format, stdout;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (historyLength < 1)
                            historyLength = 1;
                        log = logger.child({ methodName: "getCommitOnlyHistory" });
                        format = new git_format_builder_1.GitFormatBuilder().withCommit();
                        return [4 /*yield*/, this.executeGit("log -n " + historyLength + " --format=" + format, handlerParams, log)];
                    case 1:
                        stdout = _a.sent();
                        return [2 /*return*/, this.logParse(stdout.toString())];
                }
            });
        });
    };
    /**
     *
     * @param gitArgs full git command, without the 'git' prefix, e.g. log -n 1
     * @param handlerParams
     * @param log
     */
    Git.prototype.executeGit = function (gitArgs, handlerParams, log) {
        var _this = this;
        var gitPath = this.getGitPath();
        return new Promise(function (resolve, reject) {
            try {
                log.info('GIT: invoking git ' + gitArgs);
                log.info('GIT PATH: ' + gitPath);
                var cmd = gitPath + " " + gitArgs;
                child_process_1.exec(cmd, {
                    cwd: handlerParams.buildArguments.workspacepath,
                    maxBuffer: MAX_BUFFER_VALUE
                }, function (error, stdout, stderr) {
                    if (error !== null) {
                        _this.globalErrorHandler.setLastError(error, 'GIT log error: ');
                        log.error('GIT log error: ' + error);
                        log.error('stderr: ' + stderr);
                        return reject(error);
                    }
                    return resolve(stdout.toString().trim());
                });
            }
            catch (e) {
                _this.globalErrorHandler.setLastError(e, 'GIT execution error: ');
                reject(e);
            }
        });
    };
    Git.prototype.parseContributorsOutput = function (stdout, logger) {
        var _this = this;
        var contributors = [];
        var emailToIndexMap = {};
        var lines = stdout.toString().split('\n');
        var contributorIndex = 0;
        lines.forEach(function (line, lineIndex) {
            if (!line || line.trim().length == 0) {
                return;
            }
            line = line.trim();
            try {
                line = _this.fixUnSupportedJsonCharacters(line);
                var contributorData = JSON.parse(line);
                if (!emailToIndexMap.hasOwnProperty(contributorData.contributorEmail)) {
                    contributors.push(contributorData);
                    emailToIndexMap[contributorData.contributorEmail] = contributorIndex++;
                }
            }
            catch (err) {
                _this.logJsonParsError(logger, line, "contributor", err);
            }
        });
        var contributorsLog = {
            contributors: contributors,
            emailToIndexMap: emailToIndexMap
        };
        return contributorsLog;
    };
    Git.prototype.getGitPath = function () {
        return (this.getBuildArgs() || {}).gitPath || 'git';
    };
    Git.prototype.getBuildArgs = function () { return this.buildArgs; };
    Git.prototype.createCommitLogEntry = function (line, handlerParams) {
        var lineObject = JSON.parse(line); //TODO: Improve typing
        var authorEmail = lineObject.authorEmail;
        var authorIndex = handlerParams.emailToIndexMap[authorEmail];
        var commitLogEntry = {
            contributorIndex: authorIndex,
            commit: lineObject.commit,
            commiterDate: parseInt(lineObject.committerDate),
            authorDate: parseInt(lineObject.authorDate),
            title: lineObject.title
        };
        return commitLogEntry;
    };
    Git.prototype.logJsonParsError = function (logger, line, field, err) {
        logger.warn("Unsupported string format \"" + line + "\" .'" + field + "'  may not be shown in dashboard");
        logger.debug("Error:", err);
    };
    Git.prototype.parseBranchHistoryOutput = function (stdout, handlerParams, logger) {
        var _this = this;
        var commitHistory = [];
        var commitLog = [];
        var commitToIndexMap = {};
        var sendContributors = handlerParams.buildArguments.sendContributors;
        var lines = stdout.toString().split('\n');
        lines.forEach(function (line, index) {
            if (!line || line.trim().length == 0) {
                return;
            }
            line = line.trim();
            line = _this.fixUnSupportedJsonCharacters(line);
            try {
                var gitLogEntry = void 0;
                if (sendContributors) {
                    gitLogEntry = _this.createCommitLogEntry(line, handlerParams);
                }
                else {
                    gitLogEntry = JSON.parse(line); //How can this be a JSON-encoded object??
                }
                commitHistory.push(gitLogEntry.commit);
                commitLog.push(gitLogEntry);
                commitToIndexMap[gitLogEntry.commit] = index;
            }
            catch (err) {
                _this.logJsonParsError(logger, line, "commit-log", err);
            }
        });
        var gitLog = {
            commitHistory: commitHistory,
            commitLog: commitLog,
            commitToIndexMap: undefined
        };
        if (sendContributors) {
            gitLog.commitToIndexMap = commitToIndexMap;
        }
        return gitLog;
    };
    Git.prototype.parseCommitsPerFileOutput = function (stdout, commitToIndexMap, log) {
        var fileToCommitMap = {};
        var commits = stdout.toString().split('hash:');
        try {
            commits.forEach(function (currCommit) {
                currCommit = currCommit.trim();
                var commitData = currCommit.split('\n');
                if (commitData.length < 2) {
                    return;
                }
                var commitHash = commitData[0];
                for (var i = 1; i < commitData.length; i++) {
                    if (fileToCommitMap[commitData[i]]) {
                        fileToCommitMap[commitData[i]].push(commitToIndexMap[commitHash]);
                    }
                    else {
                        fileToCommitMap[commitData[i]] = [commitToIndexMap[commitHash]];
                    }
                }
            });
        }
        catch (e) {
            log.error("File to commit map error: " + e);
        }
        return fileToCommitMap;
    };
    Git.prototype.getHistoryLength = function (cfg, log) {
        var historyLength = 100;
        if (cfg.branchHistoryLength) {
            var v = parseInt(cfg.branchHistoryLength);
            if (v < 1 || Number.isNaN(v)) {
                log.error("An invalid value was specified for branchHistoryLength in sealights.json: " + cfg.branchHistoryLength + ". Valid values are positive integers. Defaulting to " + historyLength);
            }
            else {
                historyLength = v;
            }
        }
        return historyLength;
    };
    Git.prototype.fixUnSupportedJsonCharacters = function (entry) {
        // Backslashes ar not supported by json format so we replace it to double slashes.
        // For more info please see 'http://json.org/' (the string schema).
        return entry.replace(/\\/g, "\\\\");
    };
    Git.prototype.logParse = function (stdout) {
        //git output contains exactly one hash per line, so little processing is required.
        return stdout.split('\n').map(function (l) { return l.trim(); });
    };
    //Parse git log output from 'git diff [revision] --name-status -M100
    Git.prototype.parseGitDiffs = function (stdout) {
        return stdout.split('\n').map(function (l) {
            var parts = l.trim().split('\t');
            var fileStatus = parts[0];
            var path = parts[1];
            var newPath = parts[2];
            var diffEntry = { op: 'unknown', path: path };
            switch (fileStatus[0]) {
                case 'A':
                    diffEntry.op = "added";
                    break;
                case 'D':
                    diffEntry.op = "deleted";
                    break;
                case 'M':
                    diffEntry.op = "modified";
                    break;
                case 'R': //file renamed and possibly modified, depending on the rank (3 digits that follow the 'R'). rank = 100 means "renamed", otherwise renamed-modified
                    {
                        var rank = parseInt(fileStatus.substr(1));
                        diffEntry.op = (rank == 100) ? "renamed" : "renamed-modified";
                        diffEntry.newPath = newPath;
                    }
                    break;
            }
            return diffEntry;
        });
    };
    return Git;
}());
exports.Git = Git;
//# sourceMappingURL=git.js.map